import sys
from awsglue.transforms import *
from awsglue.utils import getResolvedOptions
from pyspark.context import SparkContext
from awsglue.context import GlueContext
from awsglue.job import Job
from awsglue.dynamicframe import DynamicFrame
from awsglue.utils import getResolvedOptions
import json

## @params: [JOB_NAME]
args = getResolvedOptions(sys.argv, ['JOB_NAME'])
# The following appears as is in the jobs document
dict1 = json.dumps({'AWS_GLUE_JOB_NAME':args['JOB_NAME'], 'AWS_GLUE_JOB_ID':args['JOB_ID'], 'AWS_GLUE_JOB_RUN_ID':args['JOB_RUN_ID'] })

sc = SparkContext()
glueContext = GlueContext(sc)
spark = glueContext.spark_session
job = Job(glueContext)
job.init(args['JOB_NAME'], args)

#########################################
### EXTRACT (READ DATA)
#########################################

# glue database and tables created from the crawler
glue_db = 'customer-db'
glue_tbl = 'customer_csv'

# Read customer data to Glue dynamic frame

# Annotations for AWS Glue Job
# "DataSource" is the source in an AWS Glue Job
## @type: DataSource
## @args: [database = "customer-db", table_name = "customer_csv"]
## @return: customer_df
## @inputs: []
customer_df = glueContext.create_dynamic_frame.from_catalog(database = glue_db, table_name = glue_tbl)

#Read address data to Glue dynamic frame
glue_tbl = 'address_csv'
## @type: DataSource
## @args: [database = "customer-db", table_name = "address_csv"]
## @return: address_df
## @inputs: []
address_df = glueContext.create_dynamic_frame.from_catalog(database = glue_db, table_name = glue_tbl)

#Read city data to Glue dynamic frame
glue_tbl = 'city_csv'
## @type: DataSource
## @args: [database = "customer-db", table_name = "city_csv"]
## @return: city_df
## @inputs: []
city_df = glueContext.create_dynamic_frame.from_catalog(database = glue_db, table_name = glue_tbl)

#Read country data to Glue dynamic frame
glue_tbl = 'country_csv'
## @type: DataSource
## @args: [database = "customer-db", table_name = "country_csv"]
## @return: country_df
## @inputs: []
country_df = glueContext.create_dynamic_frame.from_catalog(database = glue_db, table_name = glue_tbl)

########################################
### DENORMALIZE TO CUSTOMER ENTITY
########################################

## @type: Join
## @args: [keys1 = [<country_id>], keys2 = [<country_id>]]
## @return: <output>
## @inputs: [frame1 = <city_df>, frame2 = <country_df>]
# Join the city and country tables on the country_id key, dropping the country_id column afterward
cityFinal = Join.apply(city_df, country_df, 'country_id', 'country_id').drop_fields(['country_id'])

## @type: Join
## @args: [keys1 = [<address_id>], keys2 = [<address_id>]]
## @return: <custAddFinal>
## @inputs: [frame1 = <customer_df>, frame2 = <address_df>]
# Join the customer and address tables on the address_id key, dropping the address_id column afterward
custAddFinal = Join.apply(customer_df, address_df, 'address_id', 'address_id').drop_fields(['address_id'])

## @type: Join
## @args: [keys1 = [city_id], keys2 = [city_id]]
## @return: <customerdata>
## @inputs: [frame1 = <custAddFinal>, frame2 = <cityFinal>]
# Join the customer/address and the city/country tables together on the city_id key, dropping the city_id column afterward
customerdata = Join.apply(custAddFinal, cityFinal, 'city_id', 'city_id').drop_fields(['city_id'])

print('customer, address, city and country tables joined')

# Map to the final customer entity.
customer_mapping = [
  ('customer_id','bigint','customer_id','bigint'),
  ('first_name','string','first_name','string'),
  ('last_name','string','last_name','string'),
  ('email','string','email','string'),
  ('address','string','address.address1','string'),
  ('address2','string','address.address2','string'),
  ('district','string','address.district','string'),
  ('city','string','address.city','string'),
  ('postal_code','bigint','address.postal_code','bigint'),
  ('phone','bigint','phone','bigint'),
  ('country','string','address.country','string'), 
  ('create_date','string','create_date','timestamp')
  ]

# Apply a mapping to the joined tables to derive a final customer entity to write to the MarkLogic Data Hub
## @type: ApplyMapping
## @args: [mappings = <customer_mappings>]
## @return: <customer_entity>
## @inputs: [frame = <customerdata>]
customer_entity = customerdata.apply_mapping(customer_mapping)

customer_entity.show(1)
print('joined schema')
schema = customer_entity.schema
print('************* schema ****************')
customer_entity.printSchema()
print('job completed')

########################################
### WRITE CUSTOMER ENTITY TO MARKLOGIC DATA HUB
########################################

options_dataSinkTest = {
     #required options
    'secretId' : 'marklogic-spark-tutorial',
    'mlHost' : 'YOUR-DHS-INSTANCE-HOST',
    'className' : 'com.marklogic.hub.spark.sql.sources.v2.DefaultSource',
    
    #optional options
    'uriPrefix' : '/customer/marklogic-spark-tutorial/',
    'collections' : 'customer',
    'additionalexternalmetadata' : dict1,
    'sourceName' : 'Customer CSV',
    'sourceType' : 'type-customers',
}

# "DataSink" is the destination in a AWS Glue job
# Connection uses MarkLogic's Spark connector to store the data into the Staging database in a Data Hub
## @type: DataSink
## @args: [connection_type = 'marketplace.spark', connection_options = options_dataSinkTest]
## @return: []
## @inputs: []
glueContext.write_dynamic_frame.from_options(frame = customer_entity,
                                             connection_type = 'marketplace.spark',
                                             connection_options = options_dataSinkTest)
    
job.commit()    